<?php
/**
 * PHP version 5.
 *
 * Copyright (c) 2007-2010, Samurai Framework Project, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright notice,
 *       this list of conditions and the following disclaimer in the documentation
 *       and/or other materials provided with the distribution.
 *     * Neither the name of the Samurai Framework Project nor the names of its
 *       contributors may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @package    Samurai
 * @copyright  2007-2010 Samurai Framework Project
 * @link       http://samurai-fw.org/
 * @license    http://www.opensource.org/licenses/bsd-license.php The BSD License
 * @version    SVN: $Id: smarty.php 12 2010-01-12 02:13:11Z hayabusa $
 */

/**
 * Smarty用の初期化スクリプト
 *
 * Samurai_Renderer::initメソッドの中でインクルードされるので、
 * $this->Engine
 * でSmartyを参照できます。
 * 
 * @package    Samurai
 * @subpackage Config.Renderer
 * @copyright  2007-2010 Samurai Framework Project
 * @license    http://www.opensource.org/licenses/bsd-license.php The BSD License
 */

//ディレクトリ
$this->Engine->template_dir = Samurai_Config::get('directory.template');
$this->Engine->compile_dir = Samurai_Loader::getPath(Samurai_Config::get('directory.temp') . '/smarty/compile', true);
$this->Engine->cache_dir = Samurai_Loader::getPath(Samurai_Config::get('directory.temp') . '/smarty/cache', true);
$this->Engine->config_dir = Samurai_Loader::getPath(Samurai_Config::get('directory.config') . '/renderer/smarty');
$this->Engine->plugins_dir  = array();
foreach(Samurai::getSamuraiDirs() as $_samurai_dir){
    $this->Engine->plugins_dir[] = $_samurai_dir . '/' . Samurai_Config::get('directory.library') . '/Smarty/plugins';
}
$this->Engine->plugins_dir[] = 'plugins';

//その他のプロパティ
$this->Engine->error_reporting = NULL;
$this->Engine->compile_check = true;
$this->Engine->force_compile = false;
$this->Engine->caching = 0;
$this->Engine->cache_lifetime = 3600;
$this->Engine->cache_modified_check = false;
$this->Engine->left_delimiter = '{';
$this->Engine->right_delimiter = '}';

//Helper
$Helper = $this->addHelper('ErrorList', array('class'=>'Etc_Helper_Smarty_ErrorList'));

//アプリケーションによる上書用ファイルのインクルード
$application_override = sprintf('config/renderer/smarty.%s.php', SAMURAI_APPLICATION_NAME);
$application_override = Samurai_Loader::getPath($application_override);
if(Samurai_Loader::isReadable($application_override)) include($application_override);

//環境による上書用ファイルのインクルード
$environment_override = sprintf('config/renderer/smarty.%s.php', SAMURAI_ENVIRONMENT);
$environment_override = Samurai_Loader::getPath($environment_override);
if(Samurai_Loader::isReadable($environment_override)) include($environment_override);

